#ifndef COMMON_CK
#define COMMON_CK

namespace cmmn {

	using namespace System;

	extern int n;
	extern double * st;
	extern int * resp;
	extern const double coeff;

	extern const int NPoints;

	extern double ** gprob;
	extern double * probPSE, * probSGM;

	extern double minPSE, maxPSE, minJND, maxJND, minSGM, maxSGM, k_width;

	double sqr( double x );

	double PF( double x, double mu, double sgm );

	void calcPArrayMed( double * p, int n /* 0..n */, int & ip0, int & ip1, double & w0, double & w1  );

	void calcPArrayMode( double * p, int n /* 0..n */, int & ip );

	void calcPArrayHPD( double * p, int n, double confLevel, int & iL0, int & iL1, int & iR0, int & iR1  );

	double k_ave( double x, int * vy, double * vx, int n, double w );


	//
	//		HPĎvZ
	//
	class CalcHPD {
		public:	
			double * p;
			double maxH, mode, Lpoint, Rpoint;
			int n;

			CalcHPD( double * vp, int vn, double vmaxH, double vmode ){
				p = vp;
				n = vn;
				maxH = vmaxH;
				mode = vmode;
			}

			virtual ~CalcHPD(){ }

			void Init( double * vp, int vn, double vmaxH, double vmode ){
				p = vp;
				n = vn;
				maxH = vmaxH;
				mode = vmode;
			}
			double contDistri( double x );
			double cumDistri( double x );
			double Useek_point( double cum_p, double Lx, double Ux, double (*f)(double v) );
			double Dseek_point( double cum_p, double Lx, double Ux, double (*f)(double v) );
			double CalcIntvl( double h );
			void Seek95Intvl( );

	};

	ref class CalcGProb {
		public:
			double ** gprob;	//	pointer to the original gprob.
			int * resp;			//	copy of the original *resp.
			double * st;		//	copy of the original *st.
			int i;				//	the i-th row to be calculated.
			int NPoints, n;
			double minPSE, maxPSE, minSGM, maxSGM;
			double sum;

			CalcGProb(){ }
			~CalcGProb(){	DoIt();	}
			!CalcGProb(){	DoIt();	}

			void DoIt(){
				if (resp != nullptr) delete [] resp;
				if (st != nullptr) delete [] st;
			}

			void Init( double ** vgprob, int * vresp, double * vst, int vi, int vNPoints, int vn,
						double vminPSE, double vmaxPSE, double vminSGM, double vmaxSGM ){
				i = vi;
				NPoints = vNPoints;
				n = vn;
				gprob = vgprob;
				resp = new int[n];
				for (int k = 0; k < n; k++)
					resp[k] = vresp[k];
				st = new double[n];
				for (int k = 0; k < n; k++)
					st[k] = vst[k];
				minPSE = vminPSE;
				maxPSE = vmaxPSE;
				minSGM = vminSGM;
				maxSGM = vmaxSGM;
			}


			 double myLog( double x ){
				 double v;
				 if ( x <= 0.0 ){
						v = -750.0;
				 }else{
					 v = Math::Log( x );
				 }
				
				 return v;
			 }

			void DoCalc(){
					sum = 0.0;
					for (int j = 0; j <= NPoints; j++){
						double vPSE = minPSE + i * (maxPSE - minPSE) / NPoints;
						double vSGM = minSGM + j * (maxSGM - minSGM) / NPoints;
						double v = 0.0;
						for (int k = 0; k < n; k++)
							if (resp[k] == 1){
									v += myLog( PF(st[k], vPSE, vSGM) );
							}else{
									v += myLog( 1.0 - PF(st[k], vPSE, vSGM) );
							};
						sum += gprob[i][j] = Math::Exp(v);
					}

			}
	};

}

#endif
